@@ -48,6 +48,250 @@ struct pevt_stage1
 };
 
 
+static void mkdir_p (char *dir);
+
+
+static char *
+scrollback_get_filename (session *sess, char *buf, int max)
+{
+	char *net;
+
+	net = server_get_network (sess->server, FALSE);
+	if (!net)
+		return NULL;
+
+	snprintf (buf, max, "%s/scrollback/%s/%s.txt", get_xdir_fs (), net, sess->channel);
+	mkdir_p (buf);
+
+	return buf;
+}
+
+#if 0
+
+static void
+scrollback_unlock (session *sess)
+{
+	char buf[1024];
+
+	if (scrollback_get_filename (sess, buf, sizeof (buf) - 6) == NULL)
+		return;
+
+	strcat (buf, ".lock");
+	unlink (buf);
+}
+
+static gboolean
+scrollback_lock (session *sess)
+{
+	char buf[1024];
+	int fh;
+
+	if (scrollback_get_filename (sess, buf, sizeof (buf) - 6) == NULL)
+		return FALSE;
+
+	strcat (buf, ".lock");
+
+	if (access (buf, F_OK) == 0)
+		return FALSE;	/* can't get lock */
+
+	fh = open (buf, O_CREAT | O_TRUNC | O_APPEND | O_WRONLY, 0644);
+	if (fh == -1)
+		return FALSE;
+
+	return TRUE;
+}
+
+#endif
+
+void
+scrollback_close (session *sess)
+{
+	if (sess->scrollfd != -1)
+	{
+		close (sess->scrollfd);
+		sess->scrollfd = -1;
+	}
+}
+
+static char *
+file_to_buffer (char *file, int *len)
+{
+	int fh;
+	char *buf;
+	struct stat st;
+
+	fh = open (file, O_RDONLY | OFLAGS);
+	if (fh == -1)
+		return NULL;
+
+	fstat (fh, &st);
+
+	buf = malloc (st.st_size);
+	if (!buf)
+	{
+		close (fh);
+		return NULL;
+	}
+
+	if (read (fh, buf, st.st_size) != st.st_size)
+	{
+		free (buf);
+		close (fh);
+		return NULL;
+	}
+
+	*len = st.st_size;
+	return buf;
+}
+
+/* shrink the file to roughly prefs.max_lines */
+
+static void
+scrollback_shrink (session *sess)
+{
+	char file[1024];
+	char *buf;
+	int fh;
+	int lines;
+	int line;
+	int len;
+	char *p;
+
+	scrollback_close (sess);
+	sess->scrollwritten = 0;
+	lines = 0;
+
+	if (scrollback_get_filename (sess, file, sizeof (file)) == NULL)
+		return;
+
+	buf = file_to_buffer (file, &len);
+	if (!buf)
+		return;
+
+	/* count all lines */
+	p = buf;
+	while (p != buf + len)
+	{
+		if (*p == '\n')
+			lines++;
+		p++;
+	}
+
+	fh = open (file, O_CREAT | O_TRUNC | O_APPEND | O_WRONLY, 0644);
+	if (fh == -1)
+	{
+		free (buf);
+		return;
+	}
+
+	line = 0;
+	p = buf;
+	while (p != buf + len)
+	{
+		if (*p == '\n')
+		{
+			line++;
+			if (line >= lines - prefs.max_lines &&
+				 p + 1 != buf + len)
+			{
+				p++;
+				write (fh, p, len - (p - buf));
+				break;
+			}
+		}
+		p++;
+	}
+
+	close (fh);
+	free (buf);
+}
+
+static void
+scrollback_save (session *sess, char *text)
+{
+	char buf[1024];
+	time_t stamp;
+	int len;
+
+	if (sess->type == SESS_SERVER)
+		return;
+
+	if (sess->scrollfd == -1)
+	{
+		if (scrollback_get_filename (sess, buf, sizeof (buf)) == NULL)
+			return;
+
+		sess->scrollfd = open (buf, O_CREAT | O_APPEND | O_WRONLY, 0644);
+		if (sess->scrollfd == -1)
+			return;
+	}
+
+	stamp = time (0);
+	if (sizeof (stamp) == 4)	/* gcc will optimize one of these out */
+		write (sess->scrollfd, buf, snprintf (buf, sizeof (buf), "T %d ", (int)stamp));
+	else
+		write (sess->scrollfd, buf, snprintf (buf, sizeof (buf), "T %"G_GINT64_FORMAT" ", (gint64)stamp));
+
+	len = strlen (text);
+	write (sess->scrollfd, text, len);
+	if (len && text[len - 1] != '\n')
+		write (sess->scrollfd, "\n", 1);
+
+	sess->scrollwritten++;
+
+	if ((sess->scrollwritten * 2 > prefs.max_lines && prefs.max_lines > 0) ||
+       sess->scrollwritten > 32000)
+		scrollback_shrink (sess);
+}
+
+void
+scrollback_load (session *sess)
+{
+	int fh;
+	char buf[1024];
+	char *text;
+	time_t stamp;
+	int lines;
+
+	if (scrollback_get_filename (sess, buf, sizeof (buf)) == NULL)
+		return;
+
+	fh = open (buf, O_RDONLY | OFLAGS);
+	if (fh == -1)
+		return;
+
+	lines = 0;
+	while (waitline (fh, buf, sizeof buf, FALSE) != -1)
+	{
+		if (buf[0] == 'T')
+		{
+			if (sizeof (time_t) == 4)
+				stamp = strtoul (buf + 2, NULL, 10);
+			else
+				stamp = strtoull (buf + 2, NULL, 10); /* just incase time_t is 64 bits */
+			text = strchr (buf + 3, ' ');
+			if (text)
+			{
+				text = strip_color (text + 1, -1, STRIP_COLOR);
+				fe_print_text (sess, text, stamp);
+				free (text);
+			}
+			lines++;
+		}
+	}
+
+	if (lines)
+	{
+		text = ctime (&stamp);
+		text[24] = 0;	/* get rid of the \n */
+		snprintf (buf, sizeof (buf), "\n*\t%s %s\n\n", _("Loaded log from"), text);
+		fe_print_text (sess, buf, 0);
+		/*EMIT_SIGNAL (XP_TE_GENMSG, sess, "*", buf, NULL, NULL, NULL, 0);*/
+	}
+
+	close (fh);
+}
+
 void
 log_close (session *sess)
 {
@@ -225,10 +469,17 @@ log_create_pathname (char *servname, char *channame, char *netname)
 	strftime (fnametime, sizeof (fnametime), fname, tm);
 
 	/* create final path/filename */
-	snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
+#ifdef WIN32
+	if (fnametime[0] == '/' || (fnametime[0] >= 'A' && fnametime[1] == ':'))
+#else
+	if (fnametime[0] == '/')	/* is it fullpath already? */
+#endif
+		snprintf (fname, sizeof (fname), "%s", fnametime);
+	else
+		snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
 
 	/* now we need it in FileSystem encoding */
-	fs = g_filename_from_utf8 (fname, -1, 0, 0, 0);
+	fs = xchat_filename_from_utf8 (fname, -1, 0, 0, 0);
 
 	/* create all the subdirectories */
 	if (fs)
@@ -363,40 +614,142 @@ log_write (session *sess, char *text)
 	}
 }
 
+/* converts a CP1252/ISO-8859-1(5) hybrid to UTF-8                           */
+/* Features: 1. It never fails, all 00-FF chars are converted to valid UTF-8 */
+/*           2. Uses CP1252 in the range 80-9f because ISO doesn't have any- */
+/*              thing useful in this range and it helps us receive from mIRC */
+/*           3. The five undefined chars in CP1252 80-9f are replaced with   */
+/*              ISO-8859-15 control codes.                                   */
+/*           4. Handles 0xa4 as a Euro symbol ala ISO-8859-15.               */
+/*           5. Uses ISO-8859-1 (which matches CP1252) for everything else.  */
+/*           6. This routine measured 3x faster than g_convert :)            */
+
+static unsigned char *
+iso_8859_1_to_utf8 (unsigned char *text, int len, gsize *bytes_written)
+{
+	unsigned int idx;
+	unsigned char *res, *output;
+	static const unsigned short lowtable[] = /* 74 byte table for 80-a4 */
+	{
+	/* compressed utf-8 table: if the first byte's 0x20 bit is set, it
+	   indicates a 2-byte utf-8 sequence, otherwise prepend a 0xe2. */
+		0x82ac, /* 80 Euro. CP1252 from here on... */
+		0xe281, /* 81 NA */
+		0x809a, /* 82 */
+		0xe692, /* 83 */
+		0x809e, /* 84 */
+		0x80a6, /* 85 */
+		0x80a0, /* 86 */
+		0x80a1, /* 87 */
+		0xeb86, /* 88 */
+		0x80b0, /* 89 */
+		0xe5a0, /* 8a */
+		0x80b9, /* 8b */
+		0xe592, /* 8c */
+		0xe28d, /* 8d NA */
+		0xe5bd, /* 8e */
+		0xe28f, /* 8f NA */
+		0xe290, /* 90 NA */
+		0x8098, /* 91 */
+		0x8099, /* 92 */
+		0x809c, /* 93 */
+		0x809d, /* 94 */
+		0x80a2, /* 95 */
+		0x8093, /* 96 */
+		0x8094, /* 97 */
+		0xeb9c, /* 98 */
+		0x84a2, /* 99 */
+		0xe5a1, /* 9a */
+		0x80ba, /* 9b */
+		0xe593, /* 9c */
+		0xe29d, /* 9d NA */
+		0xe5be, /* 9e */
+		0xe5b8, /* 9f */
+		0xe2a0, /* a0 */
+		0xe2a1, /* a1 */
+		0xe2a2, /* a2 */
+		0xe2a3, /* a3 */
+		0x82ac  /* a4 ISO-8859-15 Euro. */
+	};
+
+	if (len == -1)
+		len = strlen (text);
+
+	/* worst case scenario: every byte turns into 3 bytes */
+	res = output = g_malloc ((len * 3) + 1);
+	if (!output)
+		return NULL;
+
+	while (len)
+	{
+		if (G_LIKELY (*text < 0x80))
+		{
+			*output = *text;	/* ascii maps directly */
+		}
+		else if (*text <= 0xa4)	/* 80-a4 use a lookup table */
+		{
+			idx = *text - 0x80;
+			if (lowtable[idx] & 0x2000)
+			{
+				*output++ = (lowtable[idx] >> 8) & 0xdf; /* 2 byte utf-8 */
+				*output = lowtable[idx] & 0xff;
+			}
+			else
+			{
+				*output++ = 0xe2;	/* 3 byte utf-8 */
+				*output++ = (lowtable[idx] >> 8) & 0xff;
+				*output = lowtable[idx] & 0xff;
+			}
+		}
+		else if (*text < 0xc0)
+		{
+			*output++ = 0xc2;
+			*output = *text;
+		}
+		else
+		{
+			*output++ = 0xc3;
+			*output = *text - 0x40;
+		}
+		output++;
+		text++;
+		len--;
+	}
+	*output = 0;	/* terminate */
+	*bytes_written = output - res;
+
+	return res;
+}
+
 char *
 text_validate (char **text, int *len)
 {
 	char *utf;
 	gsize utf_len;
-	GError *error = NULL;
 
 	/* valid utf8? */
 	if (g_utf8_validate (*text, *len, 0))
 		return NULL;
 
+#ifdef WIN32
+	if (GetACP () == 1252) /* our routine is better than iconv's 1252 */
+#else
 	if (prefs.utf8_locale)
+#endif
 		/* fallback to iso-8859-1 */
-		utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+		utf = iso_8859_1_to_utf8 (*text, *len, &utf_len);
 	else
 	{
 		/* fallback to locale */
 		utf = g_locale_to_utf8 (*text, *len, 0, &utf_len, NULL);
 		if (!utf)
-			utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+			utf = iso_8859_1_to_utf8 (*text, *len, &utf_len);
 	}
 
 	if (!utf) 
 	{
-		if (error)
-		{
-			*text = g_strdup_printf ("\0034ICONV\017 %s\n", error->message);
-			*len = strlen (*text);
-			g_error_free (error);
-		} else
-		{
-			*text = g_strdup ("%INVALID%");
-			*len = 9;
-		}
+		*text = g_strdup ("%INVALID%");
+		*len = 9;
 	} else
 	{
 		*text = utf;
@@ -430,7 +783,9 @@ PrintText (session *sess, char *text)
 	}
 
 	log_write (sess, text);
-	fe_print_text (sess, text);
+	if (prefs.text_replay)
+		scrollback_save (sess, text);
+	fe_print_text (sess, text, 0);
 
 	if (conv)
 		g_free (conv);
@@ -539,6 +894,7 @@ static char * const pevt_chanmsg_help[] = {
 static char * const pevt_privmsg_help[] = {
 	N_("Nickname"),
 	N_("The message"),
+	N_("Identified text")
 };
 
 static char * const pevt_changenick_help[] = {
@@ -603,12 +959,6 @@ static char * const pevt_channotice_help[] = {
 	N_("The message"),
 };
 
-static char * const pevt_dprivmsg_help[] = {
-	N_("Nickname"),
-	N_("The message"),
-	N_("Identified text"),
-};
-
 static char * const pevt_uchangenick_help[] = {
 	N_("Old nickname"),
 	N_("New nickname"),
@@ -821,12 +1171,13 @@ static char * const pevt_connect_help[] = {
 };
 
 static char * const pevt_sconnect_help[] = {
-	N_("PID"),
+	"PID"
 };
 
 static char * const pevt_generic_nick_help[] = {
 	N_("Nickname"),
-	N_("Server Name")
+	N_("Server Name"),
+	N_("Network")
 };
 
 static char * const pevt_chanmodes_help[] = {
@@ -1011,8 +1362,8 @@ pevent_load_defaults ()
 		if (pntevts_text[i])
 			free (pntevts_text[i]);
 
-		/* don't gettext() the blank ones */
-		if (i == XP_TE_OPENDIALOG || i == XP_TE_BEEP)
+		/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+		if (te[i].num_args & 128)
 			pntevts_text[i] = strdup (te[i].def);
 		else
 			pntevts_text[i] = strdup (_(te[i].def));
@@ -1022,7 +1373,7 @@ pevent_load_defaults ()
 void
 pevent_make_pntevts ()
 {
-	int i, m, len;
+	int i, m;
 	char out[1024];
 
 	for (i = 0; i < NUM_XP; i++)
@@ -1035,9 +1386,11 @@ pevent_make_pntevts ()
 						 _("Error parsing event %s.\nLoading default."), te[i].name);
 			fe_message (out, FE_MSG_WARN);
 			free (pntevts_text[i]);
-			len = strlen (te[i].def) + 1;
-			pntevts_text[i] = malloc (len);
-			memcpy (pntevts_text[i], te[i].def, len);
+			/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+			if (te[i].num_args & 128)
+				pntevts_text[i] = strdup (te[i].def);
+			else
+				pntevts_text[i] = strdup (_(te[i].def));
 			if (pevt_build_string (pntevts_text[i], &(pntevts[i]), &m) != 0)
 			{
 				fprintf (stderr,
@@ -1115,14 +1468,11 @@ pevent_load (char *filename)
 	int penum = 0;
 	char *ofs;
 
-	buf = malloc (1000);
 	if (filename == NULL)
-		snprintf (buf, 1000, "%s/pevents.conf", get_xdir_fs ());
-   else
-      safe_strcpy (buf, filename, 1000);
+		fd = xchat_open_file ("pevents.conf", O_RDONLY, 0, 0);
+	else
+		fd = xchat_open_file (filename, O_RDONLY, 0, XOF_FULLPATH);
 
-	fd = open (buf, O_RDONLY | OFLAGS);
-	free (buf);
 	if (fd == -1)
 		return 1;
 	if (fstat (fd, &st) != 0)
@@ -1207,7 +1557,7 @@ pevent_load (char *filename)
 static void
 pevent_check_all_loaded ()
 {
-	int i, len;
+	int i;
 
 	for (i = 0; i < NUM_XP; i++)
 	{
@@ -1216,9 +1566,11 @@ pevent_check_all_loaded ()
 			/*printf ("%s\n", te[i].name);
 			snprintf(out, sizeof(out), "The data for event %s failed to load. Reverting to defaults.\nThis may be because a new version of XChat is loading an old config file.\n\nCheck all print event texts are correct", evtnames[i]);
 			   gtkutil_simpledialog(out); */
-			len = strlen (te[i].def) + 1;
-			pntevts_text[i] = malloc (len);
-			memcpy (pntevts_text[i], te[i].def, len);
+			/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+			if (te[i].num_args & 128)
+				pntevts_text[i] = strdup (te[i].def);
+			else
+				pntevts_text[i] = strdup (_(te[i].def));
 		}
 	}
 }
@@ -1238,16 +1590,18 @@ load_text_events ()
 /*
 	CL: format_event now handles filtering of arguments:
 	1) if prefs.stripcolor is set, filter all style control codes from arguments
-	2) always strip \001 (ATTR_HIDDEN) from arguments: it is only for use in the format string itself
+	2) always strip \010 (ATTR_HIDDEN) from arguments: it is only for use in the format string itself
 */
+#define ARG_FLAG(argn) (1 << (argn))
+
 void
-format_event (session *sess, int index, char **args, char *o, int sizeofo)
+format_event (session *sess, int index, char **args, char *o, int sizeofo, unsigned int stripcolor_args)
 {
 	int len, oi, ii, numargs;
 	char *i, *ar, d, a, done_all = FALSE;
 
 	i = pntevts[index];
-	numargs = te[index].num_args;
+	numargs = te[index].num_args & 0x7f;
 
 	oi = ii = len = d = a = 0;
 	o[0] = 0;
@@ -1288,7 +1642,7 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo)
 				printf ("arg[%d] is NULL in print event\n", a + 1);
 			} else
 			{
-				if (prefs.stripcolor) len = strip_color2 (ar, -1, &o[oi], STRIP_ALL);
+				if (stripcolor_args & ARG_FLAG(a + 1)) len = strip_color2 (ar, -1, &o[oi], STRIP_ALL);
 				else len = strip_hidden_attribute (ar, &o[oi]);
 				oi += len;
 			}
@@ -1321,10 +1675,10 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo)
 }
 
 static void
-display_event (session *sess, int event, char **args)
+display_event (session *sess, int event, char **args, unsigned int stripcolor_args)
 {
 	char o[4096];
-	format_event (sess, event, args, o, sizeof (o));
+	format_event (sess, event, args, o, sizeof (o), stripcolor_args);
 	if (o[0])
 		PrintText (sess, o);
 }
@@ -1505,6 +1859,25 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 	return 0;
 }
 
+
+/* black n white(0/1) are bad colors for nicks, and we'll use color 2 for us */
+/* also light/dark gray (14/15) */
+/* 5,7,8 are all shades of yellow which happen to look dman near the same */
+
+static char rcolors[] = { 19, 20, 22, 24, 25, 26, 27, 28, 29 };
+
+static int
+color_of (char *name)
+{
+	int i = 0, sum = 0;
+
+	while (name[i])
+		sum += name[i++];
+	sum %= sizeof (rcolors) / sizeof (char);
+	return rcolors[sum];
+}
+
+
 /* called by EMIT_SIGNAL macro */
 
 void
@@ -1512,6 +1885,15 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 {
 	char *word[PDIWORDS];
 	int i;
+	unsigned int stripcolor_args = (prefs.stripcolor ? 0xFFFFFFFF : 0);
+	char tbuf[NICKLEN + 4];
+
+	if (prefs.colorednicks && (index == XP_TE_CHANACTION || index == XP_TE_CHANMSG))
+	{
+		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (a), a);
+		a = tbuf;
+		stripcolor_args &= ~ARG_FLAG(1);	/* don't strip color from this argument */
+	}
 
 	word[0] = te[index].name;
 	word[1] = (a ? a : "\000");
@@ -1528,7 +1910,19 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 
 	/* If a plugin's callback executes "/close", 'sess' may be invalid */
 	if (is_session (sess))
-		display_event (sess, index, word);
+		display_event (sess, index, word, stripcolor_args);
+}
+
+char *
+text_find_format_string (char *name)
+{
+	int i = 0;
+
+	i = pevent_find (name, &i);
+	if (i >= 0)
+		return pntevts_text[i];
+
+	return NULL;
 }
 
 int
@@ -1553,10 +1947,11 @@ pevent_save (char *fn)
 	char buf[1024];
 
 	if (!fn)
-		snprintf (buf, sizeof (buf), "%s/pevents.conf", get_xdir_fs ());
+		fd = xchat_open_file ("pevents.conf", O_CREAT | O_TRUNC | O_WRONLY,
+									 0x180, XOF_DOMODE);
 	else
-		safe_strcpy (buf, fn, sizeof (buf));
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+		fd = xchat_open_file (fn, O_CREAT | O_TRUNC | O_WRONLY, 0x180,
+									 XOF_FULLPATH | XOF_DOMODE);
 	if (fd == -1)
 	{
 		/*
@@ -1620,7 +2015,7 @@ sound_find_command (void)
 }
 
 void
-sound_play (const char *file)
+sound_play (const char *file, gboolean quiet)
 {
 	char buf[512];
 	char wavfile[512];
@@ -1631,9 +2026,6 @@ sound_play (const char *file)
 	if (!file[0])
 		return;
 
-	memset (buf, 0, sizeof (buf));
-	memset (wavfile, 0, sizeof (wavfile));
-
 #ifdef WIN32
 	/* check for fullpath, windows style */
 	if (strlen (file) > 3 &&
@@ -1649,8 +2041,9 @@ sound_play (const char *file)
 	{
 		strncpy (wavfile, file, sizeof (wavfile));
 	}
+	wavfile[sizeof (wavfile) - 1] = 0;	/* ensure termination */
 
-	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
+	file_fs = xchat_filename_from_utf8 (wavfile, -1, 0, 0, 0);
 	if (!file_fs)
 		return;
 
@@ -1681,8 +2074,11 @@ sound_play (const char *file)
 
 	} else
 	{
-		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
-		fe_message (buf, FE_MSG_ERROR);
+		if (!quiet)
+		{
+			snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
+			fe_message (buf, FE_MSG_ERROR);
+		}
 	}
 
 	g_free (file_fs);
@@ -1692,7 +2088,7 @@ void
 sound_play_event (int i)
 {
 	if (sound_files[i])
-		sound_play (sound_files[i]);
+		sound_play (sound_files[i], FALSE);
 }
 
 static void
@@ -1717,8 +2113,7 @@ sound_load ()
 
 	memset (&sound_files, 0, sizeof (char *) * (NUM_XP));
 
-	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
-	fd = open (buf, O_RDONLY | OFLAGS);
+	fd = xchat_open_file ("sound.conf", O_RDONLY, 0, 0);
 	if (fd == -1)
 		return;
 
@@ -1748,8 +2143,8 @@ sound_save ()
 	int fd, i;
 	char buf[512];
 
-	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+	fd = xchat_open_file ("sound.conf", O_CREAT | O_TRUNC | O_WRONLY, 0x180,
+								 XOF_DOMODE);
 	if (fd == -1)
 		return;
 
