@@ -1,5 +1,5 @@
 /* X-Chat
- * Copyright (C) 1998 Peter Zelezny.
+ * Copyright (C) 1998-2007 Peter Zelezny.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -292,7 +292,10 @@ menu_quick_item (char *cmd, char *label, GtkWidget * menu, int flags,
 			if (flags & XCMENU_MARKUP)
 			{
 				item = gtk_menu_item_new_with_label ("");
-				gtk_label_set_markup_with_mnemonic (GTK_LABEL (GTK_BIN (item)->child), label);
+				if (flags & XCMENU_MNEMONIC)
+					gtk_label_set_markup_with_mnemonic (GTK_LABEL (GTK_BIN (item)->child), label);
+				else
+					gtk_label_set_markup (GTK_LABEL (GTK_BIN (item)->child), label);
 			} else
 			{
 				if (flags & XCMENU_MNEMONIC)
@@ -600,7 +603,12 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 		menu_create (menu, popup_list, NULL, FALSE);
 	else
 		menu_create (menu, popup_list, str_copy, FALSE);
-	menu_add_plugin_items (menu, "\x5$NICK");
+
+	if (num_sel == 0)	/* xtext click */
+		menu_add_plugin_items (menu, "\x5$NICK", str_copy);
+	else	/* userlist treeview click */
+		menu_add_plugin_items (menu, "\x5$NICK", NULL);
+
 	menu_popup (menu, event, NULL);
 }
 
@@ -624,6 +632,12 @@ menu_topic_showhide_cb (session *sess)
 		gtk_widget_hide (sess->gui->topic_bar);
 }
 
+static void
+menu_userlist_showhide_cb (session *sess)
+{
+	mg_decide_userlist (sess, TRUE);
+}
+
 static void
 menu_ulbuttons_showhide_cb (session *sess)
 {
@@ -700,6 +714,14 @@ menu_topicbar_toggle (GtkWidget *wid, gpointer ud)
 								 prefs.topicbar);
 }
 
+static void
+menu_userlist_toggle (GtkWidget *wid, gpointer ud)
+{
+	prefs.hideuserlist = !prefs.hideuserlist;
+	menu_setting_foreach (menu_userlist_showhide_cb, MENU_ID_USERLIST,
+								 !prefs.hideuserlist);
+}
+
 static void
 menu_ulbuttons_toggle (GtkWidget *wid, gpointer ud)
 {
@@ -778,15 +800,20 @@ menu_urlmenu (GdkEventButton *event, char *url)
 	menu_quick_item_with_callback (copy_to_clipboard_cb, _("Copy Selected Link"), menu, str_copy);
 	/* custom ones from urlhandlers.conf */
 	menu_create (menu, urlhandler_list, str_copy, TRUE);
-	menu_add_plugin_items (menu, "\x4$URL");
+	menu_add_plugin_items (menu, "\x4$URL", str_copy);
 	menu_popup (menu, event, NULL);
 }
 
 static void
 menu_chan_cycle (GtkWidget * menu, char *chan)
 {
+	char tbuf[256];
+
 	if (current_sess)
-		handle_command (current_sess, "CYCLE", FALSE);
+	{
+		snprintf (tbuf, sizeof tbuf, "CYCLE %s", chan);
+		handle_command (current_sess, tbuf, FALSE);
+	}
 }
 
 static void
@@ -842,6 +869,7 @@ menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
 												 str_copy);
 	}
 
+	menu_add_plugin_items (menu, "\x5$CHAN", str_copy);
 	menu_popup (menu, event, NULL);
 }
 
@@ -979,6 +1007,12 @@ menu_close (GtkWidget * wid, gpointer none)
 	mg_close_sess (current_sess);
 }
 
+static void
+menu_quit (GtkWidget * wid, gpointer none)
+{
+	mg_open_quit_dialog (FALSE);
+}
+
 static void
 menu_search ()
 {
@@ -1272,24 +1306,32 @@ menu_dcc_chat_win (GtkWidget *wid, gpointer none)
 	fe_dcc_open_chat_win (FALSE);
 }
 
-static void
-menu_layout_cb (GtkWidget *item, gpointer none)
+void
+menu_change_layout (void)
 {
-	if (GTK_CHECK_MENU_ITEM (item)->active)
+	if (prefs.tab_layout == 0)
 	{
-		prefs.tab_layout = 0;
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TABS, 1);
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TREE, 0);
 		mg_change_layout (0);
 	} else
 	{
-		prefs.tab_layout = 2;
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TABS, 0);
 		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TREE, 1);
 		mg_change_layout (2);
 	}
 }
 
+static void
+menu_layout_cb (GtkWidget *item, gpointer none)
+{
+	prefs.tab_layout = 2;
+	if (GTK_CHECK_MENU_ITEM (item)->active)
+		prefs.tab_layout = 0;
+
+	menu_change_layout ();
+}
+
 static void
 menu_apply_metres_cb (session *sess)
 {
@@ -1364,37 +1406,38 @@ static struct mymenu mymenu[] = {
 #define CLOSE_OFFSET (13)
 	{0, menu_close, GTK_STOCK_CLOSE, M_MENUSTOCK, 0, 0, 1, GDK_w},
 	{0, 0, 0, M_SEP, 0, 0, 0},
-	{N_("_Quit"), mg_safe_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 0, 1, GDK_q},	/* 15 */
+	{N_("_Quit"), menu_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 0, 1, GDK_q},	/* 15 */
 
 	{N_("_View"), 0, 0, M_NEWMENU, 0, 0, 1},
 #define MENUBAR_OFFSET (17)
-	{N_("_Menubar"), menu_bar_toggle_cb, 0, M_MENUTOG, MENU_ID_MENUBAR, 0, 1, GDK_F9},
-	{N_("_Topicbar"), menu_topicbar_toggle, 0, M_MENUTOG, MENU_ID_TOPICBAR, 0, 1},
-	{N_("_Userlist Buttons"), menu_ulbuttons_toggle, 0, M_MENUTOG, MENU_ID_ULBUTTONS, 0, 1},
+	{N_("_Menu Bar"), menu_bar_toggle_cb, 0, M_MENUTOG, MENU_ID_MENUBAR, 0, 1, GDK_F9},
+	{N_("_Topic Bar"), menu_topicbar_toggle, 0, M_MENUTOG, MENU_ID_TOPICBAR, 0, 1},
+	{N_("_User List"), menu_userlist_toggle, 0, M_MENUTOG, MENU_ID_USERLIST, 0, 1, GDK_F7},
+	{N_("U_serlist Buttons"), menu_ulbuttons_toggle, 0, M_MENUTOG, MENU_ID_ULBUTTONS, 0, 1},
 	{N_("M_ode Buttons"), menu_cmbuttons_toggle, 0, M_MENUTOG, MENU_ID_MODEBUTTONS, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0, 0},
-	{N_("_Layout"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 22 */
-#define TABS_OFFSET (23)
+	{N_("_Channel Switcher"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 23 */
+#define TABS_OFFSET (24)
 		{N_("_Tabs"), menu_layout_cb, 0, M_MENURADIO, MENU_ID_LAYOUT_TABS, 0, 1},
 		{N_("T_ree"), 0, 0, M_MENURADIO, MENU_ID_LAYOUT_TREE, 0, 1},
 		{0, 0, 0, M_END, 0, 0, 0},
-	{N_("_Network Meters"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 26 */
-#define METRE_OFFSET (27)
+	{N_("_Network Meters"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 27 */
+#define METRE_OFFSET (28)
 		{N_("Off"), menu_metres_off, 0, M_MENURADIO, 0, 0, 1},
 		{N_("Graph"), menu_metres_graph, 0, M_MENURADIO, 0, 0, 1},
 		{N_("Text"), menu_metres_text, 0, M_MENURADIO, 0, 0, 1},
 		{N_("Both"), menu_metres_both, 0, M_MENURADIO, 0, 0, 1},
-		{0, 0, 0, M_END, 0, 0, 0},	/* 31 */
+		{0, 0, 0, M_END, 0, 0, 0},	/* 32 */
 
 	{N_("_Server"), 0, 0, M_NEWMENU, 0, 0, 1},
 	{N_("_Disconnect"), menu_disconnect, GTK_STOCK_DISCONNECT, M_MENUSTOCK, MENU_ID_DISCONNECT, 0, 1},
 	{N_("_Reconnect"), menu_reconnect, GTK_STOCK_CONNECT, M_MENUSTOCK, MENU_ID_RECONNECT, 0, 1},
 	{N_("Join Channel..."), menu_join, GTK_STOCK_JUMP_TO, M_MENUSTOCK, MENU_ID_JOIN, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0, 0},
-#define AWAY_OFFSET (37)
+#define AWAY_OFFSET (38)
 	{N_("Marked Away"), menu_away, 0, M_MENUTOG, MENU_ID_AWAY, 0, 1, GDK_a},
 
-	{N_("_Usermenu"), 0, 0, M_NEWMENU, MENU_ID_USERMENU, 0, 1},	/* 38 */
+	{N_("_Usermenu"), 0, 0, M_NEWMENU, MENU_ID_USERMENU, 0, 1},	/* 39 */
 
 	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 0, 1},
 	{N_("_Preferences"), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 0, 1},
@@ -1409,7 +1452,7 @@ static struct mymenu mymenu[] = {
 		{N_("User Commands..."), menu_usercommands, 0, M_MENUITEM, 0, 0, 1},
 		{N_("Userlist Buttons..."), menu_ulbuttons, 0, M_MENUITEM, 0, 0, 1},
 		{N_("Userlist Popup..."), menu_ulpopup, 0, M_MENUITEM, 0, 0, 1},
-		{0, 0, 0, M_END, 0, 0, 0},		/* 51 */
+		{0, 0, 0, M_END, 0, 0, 0},		/* 52 */
 
 	{N_("_Window"), 0, 0, M_NEWMENU, 0, 0, 1},
 	{N_("Ban List..."), menu_banlist, 0, M_MENUITEM, 0, 0, 1},
@@ -1420,15 +1463,16 @@ static struct mymenu mymenu[] = {
 	{N_("Ignore List..."), ignore_gui_open, 0, M_MENUITEM, 0, 0, 1},
 	{N_("Notify List..."), notify_opengui, 0, M_MENUITEM, 0, 0, 1},
 	{N_("Plugins and Scripts..."), menu_pluginlist, 0, M_MENUITEM, 0, 0, 1},
-	{N_("Raw Log..."), menu_rawlog, 0, M_MENUITEM, 0, 0, 1},	/* 61 */
+	{N_("Raw Log..."), menu_rawlog, 0, M_MENUITEM, 0, 0, 1},	/* 62 */
 	{N_("URL Grabber..."), url_opengui, 0, M_MENUITEM, 0, 0, 1},
 	{0, 0, 0, M_SEP, 0, 0, 0},
 	{N_("Reset Marker Line"), menu_resetmarker, 0, M_MENUITEM, 0, 0, 1, GDK_m},
 	{N_("C_lear Text"), menu_flushbuffer, GTK_STOCK_CLEAR, M_MENUSTOCK, 0, 0, 1, GDK_l},
+#define SEARCH_OFFSET 67
 	{N_("Search Text..."), menu_search, GTK_STOCK_FIND, M_MENUSTOCK, 0, 0, 1, GDK_f},
 	{N_("Save Text..."), menu_savebuffer, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 0, 1},
 
-	{N_("_Help"), 0, 0, M_NEWMENU, 0, 0, 1},	/* 68 */
+	{N_("_Help"), 0, 0, M_NEWMENU, 0, 0, 1},	/* 69 */
 	{N_("_Contents"), menu_docs, GTK_STOCK_HELP, M_MENUSTOCK, 0, 0, 1, GDK_F1},
 	{N_("_About"), menu_about, GTK_STOCK_ABOUT, M_MENUSTOCK, 0, 0, 1},
 
@@ -1541,7 +1585,7 @@ menu_find (GtkWidget *menu, char *path, char *label)
 }
 
 static void
-menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *))
+menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *, char *))
 {
 	GSList *list = sess_list;
 	int tabdone = FALSE;
@@ -1556,7 +1600,7 @@ menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *))
 		/* do it only once for tab sessions, since they share a GUI */
 		if (!sess->gui->is_tab || !tabdone)
 		{
-			callback (sess->gui->menu, me);
+			callback (sess->gui->menu, me, NULL);
 			if (sess->gui->is_tab)
 				tabdone = TRUE;
 		}
@@ -1565,7 +1609,7 @@ menu_foreach_gui (menu_entry *me, void (*callback) (GtkWidget *, menu_entry *))
 }
 
 static void
-menu_update_cb (GtkWidget *menu, menu_entry *me)
+menu_update_cb (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item;
 
@@ -1585,34 +1629,32 @@ menu_radio_cb (GtkCheckMenuItem *item, menu_entry *me)
 {
 	me->state = 0;
 	if (item->active)
-	{
 		me->state = 1;
-		if (me->cmd)
-			handle_command (current_sess, me->cmd, FALSE);
-	}
 
 	/* update the state, incase this was changed via right-click. */
 	/* This will update all other windows and menu bars */
 	menu_foreach_gui (me, menu_update_cb);
+
+	if (me->state && me->cmd)
+		handle_command (current_sess, me->cmd, FALSE);
 }
 
 /* toggle state changed via mouse click */
 static void
 menu_toggle_cb (GtkCheckMenuItem *item, menu_entry *me)
 {
+	me->state = 0;
 	if (item->active)
-	{
 		me->state = 1;
-		handle_command (current_sess, me->cmd, FALSE);
-	} else
-	{
-		me->state = 0;
-		handle_command (current_sess, me->ucmd, FALSE);
-	}
 
 	/* update the state, incase this was changed via right-click. */
 	/* This will update all other windows and menu bars */
 	menu_foreach_gui (me, menu_update_cb);
+
+	if (me->state)
+		handle_command (current_sess, me->cmd, FALSE);
+	else
+		handle_command (current_sess, me->ucmd, FALSE);
 }
 
 static GtkWidget *
@@ -1682,7 +1724,7 @@ menu_add_toggle (GtkWidget *menu, menu_entry *me)
 }
 
 static GtkWidget *
-menu_add_item (GtkWidget *menu, menu_entry *me)
+menu_add_item (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item = NULL;
 	char *path = me->path + me->root_offset;
@@ -1691,7 +1733,7 @@ menu_add_item (GtkWidget *menu, menu_entry *me)
 		menu = menu_find_path (menu, path);
 	if (menu)
 	{
-		item = menu_quick_item (me->cmd, me->label, menu, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, 0, me->icon);
+		item = menu_quick_item (me->cmd, me->label, menu, me->markup ? XCMENU_MARKUP|XCMENU_MNEMONIC : XCMENU_MNEMONIC, target, me->icon);
 		menu_reorder (GTK_MENU (menu), item, me->pos);
 	}
 	return item;
@@ -1711,13 +1753,13 @@ menu_add_sub (GtkWidget *menu, menu_entry *me)
 		pos = me->pos;
 		if (pos < 0)	/* position offset from end/bottom */
 			pos = g_list_length (GTK_MENU_SHELL (menu)->children) + pos;
-		menu_quick_sub (me->label, menu, &item, me->markup ? XCMENU_MARKUP : XCMENU_MNEMONIC, pos);
+		menu_quick_sub (me->label, menu, &item, me->markup ? XCMENU_MARKUP|XCMENU_MNEMONIC : XCMENU_MNEMONIC, pos);
 	}
 	return item;
 }
 
 static void
-menu_del_cb (GtkWidget *menu, menu_entry *me)
+menu_del_cb (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item = menu_find (menu, me->path + me->root_offset, me->label);
 	if (item)
@@ -1725,16 +1767,17 @@ menu_del_cb (GtkWidget *menu, menu_entry *me)
 }
 
 static void
-menu_add_cb (GtkWidget *menu, menu_entry *me)
+menu_add_cb (GtkWidget *menu, menu_entry *me, char *target)
 {
 	GtkWidget *item;
+	GtkAccelGroup *accel_group;
 
 	if (me->group)	/* have a group name? Must be a radio item */
 		item = menu_add_radio (menu, me);
 	else if (me->ucmd)	/* have unselect-cmd? Must be a toggle item */
 		item = menu_add_toggle (menu, me);
 	else if (me->cmd || !me->label)	/* label=NULL for separators */
-		item = menu_add_item (menu, me);
+		item = menu_add_item (menu, me, target);
 	else
 		item = menu_add_sub (menu, me);
 
@@ -1742,9 +1785,12 @@ menu_add_cb (GtkWidget *menu, menu_entry *me)
 	{
 		gtk_widget_set_sensitive (item, me->enable);
 		if (me->key)
-			gtk_widget_add_accelerator (item, "activate",
-											g_object_get_data (G_OBJECT (menu), "accel"),
-											me->key, me->modifier, GTK_ACCEL_VISIBLE);
+		{
+			accel_group = g_object_get_data (G_OBJECT (menu), "accel");
+			if (accel_group)	/* popup menus don't have them */
+				gtk_widget_add_accelerator (item, "activate", accel_group, me->key,
+													 me->modifier, GTK_ACCEL_VISIBLE);
+		}
 	}
 }
 
@@ -1790,13 +1836,13 @@ menu_add_plugin_mainmenu_items (GtkWidget *menu)
 	{
 		me = list->data;
 		if (me->is_main)
-			menu_add_cb (menu, me);
+			menu_add_cb (menu, me, NULL);
 		list = list->next;
 	}
 }
 
 void
-menu_add_plugin_items (GtkWidget *menu, char *root)
+menu_add_plugin_items (GtkWidget *menu, char *root, char *target)
 {
 	GSList *list;
 	menu_entry *me;
@@ -1806,7 +1852,7 @@ menu_add_plugin_items (GtkWidget *menu, char *root)
 	{
 		me = list->data;
 		if (!me->is_main && !strncmp (me->path, root + 1, root[0]))
-			menu_add_cb (menu, me);
+			menu_add_cb (menu, me, target);
 		list = list->next;
 	}
 }
@@ -1846,8 +1892,9 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 	/* set the initial state of toggles */
 	mymenu[MENUBAR_OFFSET].state = !prefs.hidemenu;
 	mymenu[MENUBAR_OFFSET+1].state = prefs.topicbar;
-	mymenu[MENUBAR_OFFSET+2].state = prefs.userlistbuttons;
-	mymenu[MENUBAR_OFFSET+3].state = prefs.chanmodebuttons;
+	mymenu[MENUBAR_OFFSET+2].state = !prefs.hideuserlist;
+	mymenu[MENUBAR_OFFSET+3].state = prefs.userlistbuttons;
+	mymenu[MENUBAR_OFFSET+4].state = prefs.chanmodebuttons;
 
 	mymenu[AWAY_OFFSET].state = away;
 
@@ -1889,7 +1936,10 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 		if (key_theme)
 		{
 			if (!strcasecmp (key_theme, "Emacs"))
+			{
 				close_mask = GDK_SHIFT_MASK | GDK_CONTROL_MASK;
+				mymenu[SEARCH_OFFSET].key = 0;
+			}
 			g_free (key_theme);
 		}
 	}
